@@ -34,7 +34,8 @@ from ..logger import (
     get_logger,
     set_global_verbosity_level,
     DEFAULT_LOG_FILE,
-    set_global_json_output)
+    set_global_json_output,
+    set_global_extended_view)
 from ..filters_utils import (get_filter_rules,
                              create_labels_filter_rules_list,
                              create_attributes_filter_rules_list)
@@ -386,6 +387,14 @@ def set_json(ctx, param, value):
 def set_format(ctx, param, value):
     if value == 'json':
         set_global_json_output(True)
+    elif value == 'extended':
+        set_global_extended_view(True)
+    return value
+
+
+def set_extended_view(ctx, param, value):
+    if value is not None:
+        set_global_extended_view(value)
     return value
 
 
@@ -898,6 +907,15 @@ class Options(object):
             is_flag=True,
             help=helptexts.PREVIEW)
 
+        self.extended_view = click.option(
+            '-x',
+            '--extended-view',
+            is_flag=True,
+            expose_value=False,
+            default=None,
+            help=helptexts.EXTENDED_VIEW,
+            callback=set_extended_view)
+
         self.dont_update_plugins = click.option(
             '--dont-update-plugins',
             is_flag=True,
@@ -1765,6 +1783,12 @@ class Options(object):
             help=helptexts.SEARCH_NAME
         )
 
+        self.dependencies_of = click.option(
+            '--dependencies-of',
+            callback=validate_value_not_empty,
+            help=helptexts.DEPENDENCIES_OF
+        )
+
     def common_options(self, f):
         """A shorthand for applying commonly used arguments.
 
@@ -1776,6 +1800,16 @@ class Options(object):
             f = arg(f)
         return f
 
+    def local_common_options(self, f):
+        """A shorthand for applying commonly used arguments for local profiles.
+
+        Similar to common_options, but doesn't apply the options that are only
+        relevant to managers.
+        """
+        for arg in [self.json, self.verbose(), self.format, self.quiet()]:
+            f = arg(f)
+        return f
+
     def parse_comma_separated(self, ctx, param, value):
         """Callback for parsing multiple comma-separated arguments.
 
@@ -1923,6 +1957,14 @@ class Options(object):
             default=DEFAULT_BLUEPRINT_PATH,
             help=helptexts.BLUEPRINT_FILENAME + extra_message)
 
+    @staticmethod
+    def blueprint_icon_path(required=False):
+        return click.option(
+            '-i',
+            '--icon-path',
+            required=required,
+            help=helptexts.BLUEPRINT_ICON_PATH)
+
     @staticmethod
     def workflow_id(default=None, required=False):
         return click.option(
@@ -2258,6 +2300,14 @@ class Options(object):
             required=False,
             help=help)
 
+    @staticmethod
+    def new_username(required=True):
+        return click.option(
+            '-s',
+            '--username',
+            required=required,
+            help=helptexts.SET_USERNAME)
+
     @staticmethod
     def _filter_rules(f, resource):
         help_text = (helptexts.DEPLOYMENTS_ATTRS_FILTER_RULES if
