@@ -39,7 +39,7 @@ from ..utils import prettify_client_error, get_visibility, validate_visibility
 from ..labels_utils import (add_labels,
                             delete_labels,
                             list_labels,
-                            modify_resource_labels)
+                            serialize_resource_labels)
 from .. import filters_utils
 from .summary import BASE_SUMMARY_FIELDS, structure_summary_results
 
@@ -56,32 +56,7 @@ BLUEPRINTS_SUMMARY_FIELDS = BASE_SUMMARY_FIELDS
 @cfy.group(name='blueprints')
 @cfy.options.common_options
 def blueprints():
-    """Handle blueprints on the manager
-    """
-    pass
-
-
-@blueprints.command(name='validate',
-                    short_help='Validate a blueprint')
-@cfy.argument('blueprint-path')
-@cfy.options.common_options
-@cfy.pass_logger
-def validate_blueprint(blueprint_path, logger):
-    """Validate a blueprint
-
-    `BLUEPRINT_PATH` is the path of the blueprint to validate.
-    """
-    logger.info('Validating blueprint: {0}'.format(blueprint_path))
-    try:
-        resolver = config.get_import_resolver()
-        validate_version = config.is_validate_definitions_version()
-        parse_from_path(
-            dsl_file_path=blueprint_path,
-            resolver=resolver,
-            validate_version=validate_version)
-    except DSLParsingException as ex:
-        raise CloudifyCliError('Failed to validate blueprint: {0}'.format(ex))
-    logger.info('Blueprint validated successfully')
+    """Handle blueprints on the manager"""
 
 
 @blueprints.command(name='upload',
@@ -89,6 +64,7 @@ def validate_blueprint(blueprint_path, logger):
 @cfy.argument('blueprint-path')
 @cfy.options.blueprint_id(validate=True)
 @cfy.options.blueprint_filename()
+@cfy.options.blueprint_icon_path()
 @cfy.options.async_upload
 @cfy.options.labels
 @cfy.options.validate
@@ -104,6 +80,7 @@ def upload(ctx,
            blueprint_path,
            blueprint_id,
            blueprint_filename,
+           icon_path,
            async_upload,
            labels,
            validate,
@@ -123,7 +100,7 @@ def upload(ctx,
     client.license.check()
     utils.explicit_tenant_name_message(tenant_name, logger)
     processed_blueprint_path = blueprint.get(
-        blueprint_path, blueprint_filename)
+        blueprint_path, blueprint_filename, icon_path)
 
     # Take into account that `blueprint.get` might not return a URL
     # instead of a blueprint file (archive files are not locally downloaded)
@@ -155,6 +132,8 @@ def upload(ctx,
             async_upload=True,
             labels=labels
         )
+        if icon_path:
+            client.blueprints.upload_icon(blueprint_id, icon_path)
     else:
         try:
             if validate:
@@ -240,7 +219,7 @@ def delete(blueprint_id, force, logger, client, tenant_name):
     logger.info('Blueprint deleted')
 
 
-@cfy.command(name='list', short_help='List blueprints')
+@blueprints.command(name='list', short_help='List blueprints')
 @cfy.options.filter_id
 @cfy.options.blueprint_filter_rules
 @cfy.options.sort_by()
@@ -255,6 +234,7 @@ def delete(blueprint_id, force, logger, client, tenant_name):
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
+@cfy.options.extended_view
 def manager_list(filter_id,
                  filter_rules,
                  sort_by,
@@ -291,7 +271,7 @@ def manager_list(filter_id,
         filter_id=filter_id
     )
     blueprints = [trim_description(b) for b in blueprints_list]
-    modify_resource_labels(blueprints)
+    serialize_resource_labels(blueprints)
     print_data(BLUEPRINT_COLUMNS, blueprints, 'Blueprints:')
 
     total = blueprints_list.metadata.pagination.total
@@ -304,13 +284,6 @@ def manager_list(filter_id,
     logger.info(base_str)
 
 
-@cfy.command(name='list', short_help='List blueprints')
-@cfy.pass_logger
-def local_list(logger):
-    blueprints = local.list_blueprints()
-    print_data(BASE_BLUEPRINT_COLUMNS, blueprints, 'Blueprints:')
-
-
 @blueprints.command(name='get',
                     short_help='Retrieve blueprint information [manager only]')
 @cfy.argument('blueprint-id')
@@ -319,6 +292,7 @@ def local_list(logger):
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
+@cfy.options.extended_view
 def get(blueprint_id, logger, client, tenant_name):
     """Retrieve information for a specific blueprint
 
@@ -332,6 +306,13 @@ def get(blueprint_id, logger, client, tenant_name):
     blueprint_dict['#deployments'] = len(deployments)
     columns = BLUEPRINT_COLUMNS + ['#deployments']
     blueprint_metadata = blueprint_dict['plan']['metadata'] or {}
+    blueprint_plugins = {k: [p for p in blueprint_dict['plan'][k]
+                             if p['package_name'] and p['package_version']]
+                         for k in ['deployment_plugins_to_install',
+                                   'workflow_plugins_to_install',
+                                   'host_agent_plugins_to_install']
+                         if k in blueprint_dict['plan']
+                         and blueprint_dict['plan'][k]}
     blueprint_deployments = [d['id'] for d in deployments]
 
     if get_global_json_output():
@@ -341,6 +322,7 @@ def get(blueprint_id, logger, client, tenant_name):
         blueprint_dict['deployments'] = blueprint_deployments
         print_single(columns, blueprint_dict, 'Blueprint:', max_width=50)
     else:
+        serialize_resource_labels([blueprint_dict])
         print_single(columns, blueprint_dict, 'Blueprint:', max_width=50)
 
         logger.info('Description:')
@@ -353,6 +335,23 @@ def get(blueprint_id, logger, client, tenant_name):
                 logger.info('\t{0}: {1}'.format(property_name, property_value))
             logger.info('')
 
+        if blueprint_plugins:
+            plugins_dict = {}
+            for plugin_key, plugins in blueprint_plugins.items():
+                plugin_purpose = plugin_key.partition('_')[0]
+                for plugin in plugins:
+                    plugin_id = '{0}=={1}'.format(plugin['package_name'],
+                                                  plugin['package_version'])
+                    if plugin_id in plugins_dict:
+                        plugins_dict[plugin_id].append(plugin_purpose)
+                    else:
+                        plugins_dict[plugin_id] = [plugin_purpose]
+            logger.info('Plugins:')
+            for plugin, plugin_purpose in plugins_dict.items():
+                logger.info('\t{0} ({1})'.format(plugin,
+                                                 ', '.join(plugin_purpose)))
+            logger.info('')
+
         logger.info('Existing deployments:')
         logger.info('{0}\n'.format(json.dumps(blueprint_deployments)))
 
@@ -365,6 +364,7 @@ def get(blueprint_id, logger, client, tenant_name):
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
+@cfy.options.extended_view
 def inputs(blueprint_id, logger, client, tenant_name):
     """Retrieve inputs for a specific blueprint
 
@@ -445,24 +445,6 @@ def create_requirements(blueprint_path, output_path, logger):
             logger.info(requirement)
 
 
-@blueprints.command(name='install-plugins',
-                    short_help='Install plugins [locally]')
-@cfy.argument('blueprint-path', type=click.Path(exists=True))
-@cfy.options.common_options
-@cfy.assert_local_active
-@cfy.pass_logger
-def install_plugins(blueprint_path, logger):
-    """Install the necessary plugins for a given blueprint in the
-    local environment.
-
-    Currently only supports passing the YAML of the blueprint directly.
-
-    `BLUEPRINT_PATH` is the path to the blueprint to install plugins for.
-    """
-    logger.info('Installing plugins...')
-    local._install_plugins(blueprint_path=blueprint_path)
-
-
 @blueprints.command(name='set-global',
                     short_help="Set the blueprint's visibility to global")
 @cfy.argument('blueprint-id')
@@ -546,6 +528,47 @@ def summary(target_field, sub_field, logger, client, tenant_name,
     )
 
 
+@blueprints.command(name='set-icon',
+                    short_help="Set or remove blueprint's icon")
+@cfy.argument('blueprint-id')
+@cfy.options.blueprint_icon_path()
+@cfy.assert_manager_active()
+@cfy.pass_client()
+@cfy.pass_logger
+def set_icon(blueprint_id, icon_path, logger, client):
+    """Set an icon which will be used to describe/identify the blueprint.
+    In case `-i [ICON_PATH]` is provided, the [ICON_PATH] should point to
+    a valid PNG image. If this parameter is omitted, the icon will be removed
+    from the blueprint's resources.
+    """
+    status_codes = [400, 403, 404]
+    with prettify_client_error(status_codes, logger):
+        if icon_path:
+            client.blueprints.upload_icon(blueprint_id, icon_path)
+            logger.info('Blueprint `{0}` has a new icon set.'
+                        .format(blueprint_id))
+        else:
+            client.blueprints.remove_icon(blueprint_id)
+            logger.info('Blueprint `{0}` has its icon removed.'
+                        .format(blueprint_id))
+
+
+@blueprints.command(name='set-owner',
+                    short_help="Change blueprint's ownership")
+@cfy.argument('blueprint-id')
+@cfy.options.new_username()
+@cfy.options.tenant_name(required=False, resource_name_for_help='secret')
+@cfy.assert_manager_active()
+@cfy.pass_client(use_tenant_in_header=True)
+@cfy.pass_logger
+def set_owner(blueprint_id, username, tenant_name, logger, client):
+    """Set a new owner for the blueprint."""
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    bp = client.blueprints.update(blueprint_id, {'creator': username})
+    logger.info('Blueprint `%s` is now owned by user `%s`.',
+                blueprint_id, bp.get('created_by'))
+
+
 @blueprints.group(name='labels',
                   short_help="Handle a blueprint's labels")
 @cfy.options.common_options
@@ -753,3 +776,62 @@ def delete_deployments_filter(filter_id, tenant_name, logger, client):
                                 tenant_name,
                                 logger,
                                 client.blueprints_filters)
+
+
+@cfy.group(name='blueprints')
+@cfy.options.common_options
+def local_blueprints():
+    """Handle local blueprints"""
+
+
+@local_blueprints.command(name='list', short_help='List blueprints')
+@cfy.options.local_common_options
+@cfy.pass_logger
+@cfy.options.extended_view
+def local_list(logger):
+    blueprints = local.list_blueprints()
+    print_data(BASE_BLUEPRINT_COLUMNS, blueprints, 'Blueprints:')
+
+
+@local_blueprints.command(name='install-plugins',
+                          short_help='Install plugins [locally]')
+@cfy.argument('blueprint-path', type=click.Path(exists=True))
+@cfy.options.common_options
+@cfy.assert_local_active
+@cfy.pass_logger
+def install_plugins(blueprint_path, logger):
+    """Install the necessary plugins for a given blueprint in the
+       local environment.
+
+    Currently only supports passing the YAML of the blueprint directly.
+
+    `BLUEPRINT_PATH` is the path to the blueprint to install plugins for.
+    """
+    logger.info('Installing plugins...')
+    local._install_plugins(blueprint_path=blueprint_path)
+
+
+@click.command(name='validate', short_help='Validate a blueprint')
+@cfy.argument('blueprint-path')
+@cfy.options.common_options
+@cfy.pass_logger
+def validate_blueprint(blueprint_path, logger):
+    """Validate a blueprint
+
+    `BLUEPRINT_PATH` is the path of the blueprint to validate.
+    """
+    logger.info('Validating blueprint: {0}'.format(blueprint_path))
+    try:
+        resolver = config.get_import_resolver()
+        validate_version = config.is_validate_definitions_version()
+        parse_from_path(
+            dsl_file_path=blueprint_path,
+            resolver=resolver,
+            validate_version=validate_version)
+    except DSLParsingException as ex:
+        raise CloudifyCliError('Failed to validate blueprint: {0}'.format(ex))
+    logger.info('Blueprint validated successfully')
+
+
+blueprints.add_command(validate_blueprint)
+local_blueprints.add_command(validate_blueprint)
