@@ -41,7 +41,12 @@ from ..table import (
     print_list
 )
 from ..cli import cfy, helptexts
-from ..logger import get_events_logger, get_global_json_output, output
+from ..logger import (
+    get_events_logger,
+    get_global_json_output,
+    output,
+    get_global_extended_view
+)
 from .. import env, execution_events_fetcher, utils
 from ..constants import DEFAULT_BLUEPRINT_PATH, DELETE_DEP
 from ..exceptions import (CloudifyCliError,
@@ -56,7 +61,7 @@ from ..labels_utils import (add_labels,
                             get_output_resource_labels,
                             get_printable_resource_labels,
                             list_labels,
-                            modify_resource_labels)
+                            serialize_resource_labels)
 
 from .. import filters_utils
 from .summary import BASE_SUMMARY_FIELDS, structure_summary_results
@@ -67,6 +72,10 @@ DEPLOYMENT_COLUMNS = [
     'visibility', 'tenant_name', 'created_by', 'site_name', 'labels',
     'deployment_status', 'installation_status'
 ]
+EXTENDED_DEPLOYMENT_COLUMNS = DEPLOYMENT_COLUMNS + [
+    'sub_services_count', 'sub_services_status', 'sub_environments_count',
+    'sub_environments_status'
+]
 DEPLOYMENT_UPDATE_COLUMNS = [
     'id', 'deployment_id', 'tenant_name', 'state', 'execution_id',
     'created_at', 'visibility', 'old_blueprint_id', 'new_blueprint_id'
@@ -111,9 +120,7 @@ SCHEDULE_TABLE_COLUMNS = ['id', 'deployment_id', 'workflow_id', 'created_at',
 @cfy.group(name='deployments')
 @cfy.options.common_options
 def deployments():
-    """Handle deployments on the Manager
-    """
-    pass
+    """Handle deployments on the Manager"""
 
 
 def _print_single_update(deployment_update_dict,
@@ -192,7 +199,7 @@ def _print_single_update(deployment_update_dict,
                    'The following labels will be created: ')
 
 
-@cfy.command(name='list', short_help='List deployments [manager only]')
+@deployments.command(name='list', short_help='List deployments [manager only]')
 @cfy.options.blueprint_id()
 @click.option('--group-id', '-g')
 @cfy.options.filter_id
@@ -204,12 +211,14 @@ def _print_single_update(deployment_update_dict,
 @cfy.options.all_tenants
 @cfy.options.search
 @cfy.options.search_name
+@cfy.options.dependencies_of
 @cfy.options.pagination_offset
 @cfy.options.pagination_size
 @cfy.options.common_options
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
+@cfy.options.extended_view
 def manager_list(blueprint_id,
                  group_id,
                  filter_id,
@@ -219,6 +228,7 @@ def manager_list(blueprint_id,
                  all_tenants,
                  search,
                  search_name,
+                 dependencies_of,
                  pagination_offset,
                  pagination_size,
                  logger,
@@ -246,10 +256,15 @@ def manager_list(blueprint_id,
                                           _size=pagination_size,
                                           _group_id=group_id,
                                           blueprint_id=blueprint_id,
-                                          _search_name=search_name)
-    modify_resource_labels(deployments)
+                                          _search_name=search_name,
+                                          _dependencies_of=dependencies_of)
+    serialize_resource_labels(deployments)
     total = deployments.metadata.pagination.total
-    print_data(DEPLOYMENT_COLUMNS, deployments, 'Deployments:')
+    if get_global_extended_view() or get_global_json_output():
+        columns = EXTENDED_DEPLOYMENT_COLUMNS
+    else:
+        columns = DEPLOYMENT_COLUMNS
+    print_data(columns, deployments, 'Deployments:')
 
     base_str = 'Showing {0} of {1} deployments'.format(len(deployments), total)
     if filter_rules or filter_id:
@@ -259,8 +274,8 @@ def manager_list(blueprint_id,
     logger.info(base_str)
 
 
-@cfy.command(name='history', short_help='List deployment updates '
-                                        '[manager only]')
+@deployments.command(name='history',
+                     short_help='List deployment updates [manager only]')
 @cfy.options.deployment_id()
 @cfy.options.sort_by()
 @cfy.options.descending
@@ -274,6 +289,7 @@ def manager_list(blueprint_id,
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
+@cfy.options.extended_view
 def manager_history(deployment_id,
                     sort_by,
                     descending,
@@ -312,7 +328,7 @@ def manager_history(deployment_id,
         len(deployment_updates), total))
 
 
-@cfy.command(
+@deployments.command(
     name='get-update',
     short_help='Retrieve deployment update information [manager only]'
 )
@@ -337,7 +353,8 @@ def manager_get_update(deployment_update_id, logger, client, tenant_name):
     _print_single_update(deployment_update_dict)
 
 
-@cfy.command(name='update', short_help='Update a deployment [manager only]')
+@deployments.command(
+    name='update', short_help='Update a deployment [manager only]')
 @cfy.argument('deployment-id')
 @cfy.options.blueprint_path(extra_message=' [UNSUPPORTED]')
 @cfy.options.blueprint_filename(' [UNSUPPORTED]')
@@ -495,8 +512,8 @@ def manager_update(ctx,
                             exec_id=execution.id))
 
 
-@cfy.command(name='create',
-             short_help='Create a deployment [manager only]')
+@deployments.command(name='create',
+                     short_help='Create a deployment [manager only]')
 @cfy.argument('deployment-id', required=False, callback=cfy.validate_name)
 @cfy.options.blueprint_id(required=True)
 @cfy.options.inputs
@@ -581,8 +598,8 @@ def manager_create(blueprint_id,
                 "{1}".format(deployment.display_name, deployment.id))
 
 
-@cfy.command(name='delete',
-             short_help='Delete a deployment [manager only]')
+@deployments.command(name='delete',
+                     short_help='Delete a deployment [manager only]')
 @cfy.argument('deployment-id')
 @cfy.options.force(help=helptexts.FORCE_DELETE_DEPLOYMENT)
 @cfy.options.common_options
@@ -630,8 +647,8 @@ def manager_delete(deployment_id, force, with_logs, logger, client,
     logger.info("Deployment deleted")
 
 
-@cfy.command(name='outputs',
-             short_help='Show deployment outputs [manager only]')
+@deployments.command(name='outputs',
+                     short_help='Show deployment outputs [manager only]')
 @cfy.argument('deployment-id')
 @cfy.options.common_options
 @cfy.options.tenant_name(required=False, resource_name_for_help='deployment')
@@ -652,8 +669,8 @@ def manager_outputs(deployment_id, logger, client, tenant_name):
     )
 
 
-@cfy.command(name='capabilities',
-             short_help='Show deployment capabilities [manager only]')
+@deployments.command(name='capabilities',
+                     short_help='Show deployment capabilities [manager only]')
 @cfy.argument('deployment-id')
 @cfy.options.common_options
 @cfy.options.tenant_name(required=False, resource_name_for_help='deployment')
@@ -706,8 +723,8 @@ def _present_outputs_or_capabilities(
         logger.info(values.getvalue())
 
 
-@cfy.command(name='inputs',
-             short_help='Show deployment inputs [manager only]')
+@deployments.command(name='inputs',
+                     short_help='Show deployment inputs [manager only]')
 @cfy.argument('deployment-id')
 @cfy.options.common_options
 @cfy.options.tenant_name(required=False, resource_name_for_help='deployment')
@@ -733,8 +750,10 @@ def manager_inputs(deployment_id, logger, client, tenant_name):
         logger.info(inputs_.getvalue())
 
 
-@cfy.command(name='set-visibility',
-             short_help="Set the deployment's visibility [manager only]")
+@deployments.command(
+    name='set-visibility',
+    short_help="Set the deployment's visibility [manager only]"
+)
 @cfy.argument('deployment-id')
 @cfy.options.visibility(required=True, valid_values=VISIBILITY_EXCEPT_PRIVATE)
 @cfy.options.common_options
@@ -754,28 +773,6 @@ def manager_set_visibility(deployment_id, visibility, logger, client):
                                                              visibility))
 
 
-@cfy.command(name='inputs', short_help='Show deployment inputs [locally]')
-@cfy.options.common_options
-@cfy.options.blueprint_id(required=True)
-@cfy.pass_logger
-def local_inputs(blueprint_id, logger):
-    """Display inputs for the execution
-    """
-    env = load_env(blueprint_id)
-    logger.info(json.dumps(env.plan['inputs'] or {}, sort_keys=True, indent=2))
-
-
-@cfy.command(name='outputs', short_help='Show deployment outputs [locally]')
-@cfy.options.common_options
-@cfy.options.blueprint_id(required=True)
-@cfy.pass_logger
-def local_outputs(blueprint_id, logger):
-    """Display outputs for the execution
-    """
-    env = load_env(blueprint_id)
-    logger.info(json.dumps(env.outputs() or {}, sort_keys=True, indent=2))
-
-
 @deployments.command(name='summary',
                      short_help='Retrieve summary of deployment details '
                                 '[manager only]')
@@ -820,8 +817,8 @@ def summary(target_field, sub_field, group_id, logger, client, tenant_name,
     )
 
 
-@cfy.command(name='set-site',
-             short_help="Set the deployment's site [manager only]")
+@deployments.command(name='set-site',
+                     short_help="Set the deployment's site [manager only]")
 @cfy.argument('deployment-id')
 @cfy.options.site_name
 @cfy.options.detach_site
@@ -851,6 +848,22 @@ def manager_set_site(deployment_id, site_name, detach_site, client, logger):
                                                               site_name))
 
 
+@deployments.command(name='set-owner',
+                     short_help="Change deployment's ownership")
+@cfy.argument('deployment-id')
+@cfy.options.new_username()
+@cfy.options.tenant_name(required=False, resource_name_for_help='secret')
+@cfy.assert_manager_active()
+@cfy.pass_client(use_tenant_in_header=True)
+@cfy.pass_logger
+def set_owner(deployment_id, username, tenant_name, logger, client):
+    """Set a new owner for the deployment."""
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    dep = client.deployments.set_attributes(deployment_id, creator=username)
+    logger.info('Deployment `%s` is now owned by user `%s`.',
+                deployment_id, dep.get('created_by'))
+
+
 @deployments.group(name='labels',
                    short_help="Handle a deployment's labels")
 @cfy.options.common_options
@@ -940,6 +953,7 @@ def modifications():
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
+@cfy.options.extended_view
 def list_modifications(deployment_id,
                        pagination_offset,
                        pagination_size,
@@ -1059,6 +1073,7 @@ def _format_group(g):
 @groups.command('list', short_help='List all deployment groups')
 @cfy.pass_client()
 @cfy.pass_logger
+@cfy.options.extended_view
 def groups_list(client, logger):
     groups = [_format_group(g) for g in client.deployment_groups.list()]
     print_data(DEP_GROUP_COLUMNS, groups, 'Deployment groups:')
@@ -1290,13 +1305,10 @@ def groups_update_deployments(ctx, group_id, logger, client, tenant_name,
                 execution_group.id)
 
 
-@cfy.group(name='schedule')
+@deployments.group(name='schedule')
 @cfy.options.common_options
 def schedule():
-    """
-    Handle deployments' execution scheduling [manager only]
-    """
-    pass
+    """Handle deployments' execution scheduling [manager only]"""
 
 
 @schedule.command(name='create',
@@ -1552,6 +1564,7 @@ def schedule_delete(deployment_id, schedule_id, logger, client, tenant_name):
 @cfy.options.tz
 @cfy.pass_client()
 @cfy.pass_logger
+@cfy.options.extended_view
 def schedule_list(deployment_id,
                   sort_by,
                   descending,
@@ -1613,6 +1626,7 @@ def schedule_list(deployment_id,
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
+@cfy.options.extended_view
 def schedule_get(deployment_id,
                  schedule_id,
                  preview,
@@ -1861,3 +1875,33 @@ def delete_deployments_filter(filter_id, tenant_name, logger, client):
                                 tenant_name,
                                 logger,
                                 client.deployments_filters)
+
+
+@cfy.group(name='deployments')
+@cfy.options.common_options
+def local_deployments():
+    """Handle local deployments"""
+
+
+@local_deployments.command(
+    name='inputs', short_help='Show deployment inputs [locally]')
+@cfy.options.common_options
+@cfy.options.blueprint_id(required=True)
+@cfy.pass_logger
+def local_inputs(blueprint_id, logger):
+    """Display inputs for the execution
+    """
+    env = load_env(blueprint_id)
+    logger.info(json.dumps(env.plan['inputs'] or {}, sort_keys=True, indent=2))
+
+
+@local_deployments.command(
+    name='outputs', short_help='Show deployment outputs [locally]')
+@cfy.options.common_options
+@cfy.options.blueprint_id(required=True)
+@cfy.pass_logger
+def local_outputs(blueprint_id, logger):
+    """Display outputs for the execution
+    """
+    env = load_env(blueprint_id)
+    logger.info(json.dumps(env.outputs() or {}, sort_keys=True, indent=2))
