@@ -16,7 +16,6 @@
 
 
 import os
-import shutil
 import pkg_resources
 
 from jinja2.environment import Template
@@ -33,7 +32,7 @@ from ..logger import configure_loggers
 from ..exceptions import CloudifyCliError
 
 
-@cfy.command(name='init', short_help='Initialize a working env')
+@cfy.command(name='init',)
 @cfy.argument('blueprint-path', required=False)
 @cfy.options.blueprint_filename()
 @cfy.options.blueprint_id(required=False, validate=True)
@@ -68,6 +67,7 @@ def init(blueprint_path,
     ~/.cloudify/config.yaml. For more information refer to the docs
     at http://docs.getcloudify.org
     """
+
     profile_name = 'local'
 
     if blueprint_path:
@@ -92,8 +92,8 @@ def init(blueprint_path,
             processed_blueprint_path,
             blueprint_filename
         )
-        if os.path.isdir(local.storage_dir(blueprint_id)):
-            shutil.rmtree(local.storage_dir(blueprint_id))
+        if local.blueprint_exists(blueprint_id):
+            local.remove(blueprint_id)
 
         try:
             storage = local.get_storage()
