@@ -57,6 +57,7 @@ def node_instances():
     required=False, resource_name_for_help='node-instance')
 @cfy.pass_logger
 @cfy.pass_client()
+@cfy.options.extended_view
 def get(node_instance_id, logger, client, tenant_name):
     """Retrieve information for a specific node-instance
 
@@ -92,6 +93,7 @@ def get(node_instance_id, logger, client, tenant_name):
 @cfy.options.common_options
 @cfy.pass_logger
 @cfy.pass_client()
+@cfy.options.extended_view
 def list(deployment_id,
          node_name,
          sort_by,
@@ -193,6 +195,7 @@ def summary(target_field, sub_field, logger, client, tenant_name,
                          resource_name_for_help='node-instance')
 @cfy.pass_logger
 @cfy.pass_client()
+@cfy.options.extended_view
 def update_runtime(node_instance_id, logger, client, tenant_name, properties):
     """Update the runtime properties of a specific node-instance
 
@@ -212,6 +215,7 @@ def update_runtime(node_instance_id, logger, client, tenant_name, properties):
                          resource_name_for_help='node-instance')
 @cfy.pass_logger
 @cfy.pass_client()
+@cfy.options.extended_view
 def delete_runtime(node_instance_id, logger, client, tenant_name, properties):
     """Delete specified runtime properties of a specific node-instance
 
@@ -264,9 +268,10 @@ def local(node_id, blueprint_id, logger):
 
 def _print_node_instance(node_instance):
     if get_global_json_output():
-        # For json output, make sure runtime properties are in the same object
+        # For json output, make sure the properties are in the same object
         # so that the output is a single decode-able object
-        columns = NODE_INSTANCE_COLUMNS + ['runtime_properties']
+        columns = NODE_INSTANCE_COLUMNS + \
+            ['runtime_properties', 'system_properties']
         print_single(columns, node_instance, 'Node-instance:', 50)
     else:
         print_single(NODE_INSTANCE_COLUMNS, node_instance,
@@ -274,3 +279,7 @@ def _print_node_instance(node_instance):
 
         print_details(node_instance.runtime_properties,
                       'Instance runtime properties:')
+
+        if hasattr(node_instance, 'system_properties'):
+            print_details(node_instance.system_properties,
+                          'Instance system properties:')
