@@ -39,7 +39,6 @@ from ..commands.cluster import update_profile_logic as update_cluster_profile
 
 
 EXPORTED_KEYS_DIRNAME = '.exported-ssh-keys'
-EXPORTED_SSH_KEYS_DIR = os.path.join(env.PROFILES_DIR, EXPORTED_KEYS_DIRNAME)
 PROFILE_COLUMNS = ['name', 'manager_ip', 'manager_username', 'manager_tenant',
                    'ssh_user', 'ssh_key', 'ssh_port', 'kerberos_env',
                    'rest_port', 'rest_protocol', 'rest_certificate']
@@ -47,11 +46,14 @@ CLUSTER_PROFILE_COLUMNS = PROFILE_COLUMNS[:1] + ['hostname', 'host_ip'] \
     + PROFILE_COLUMNS[2:]
 
 
+def _exported_ssh_keys_dir():
+    return os.path.join(env.PROFILES_DIR, EXPORTED_KEYS_DIRNAME)
+
+
 @cfy.group(name='profiles')
 @cfy.options.common_options
 def profiles():
-    """
-    Handle Cloudify CLI profiles
+    """Handle Cloudify CLI profiles
 
     Each profile can manage a single Cloudify manager.
 
@@ -85,6 +87,7 @@ def _format_cluster_profile(profile):
                   short_help='Retrieve current profile information')
 @cfy.options.common_options
 @cfy.pass_logger
+@cfy.options.extended_view
 def show(logger):
     """
     Shows your current active profile and it's properties
@@ -114,6 +117,7 @@ def show(logger):
                   short_help='List profiles')
 @cfy.options.common_options
 @cfy.pass_logger
+@cfy.options.extended_view
 def profiles_list(logger):
     """
     List all profiles
@@ -612,7 +616,7 @@ def export_profiles(include_keys, output_path, logger):
             _backup_ssh_key(profile)
     utils.tar(env.PROFILES_DIR, destination)
     if include_keys:
-        shutil.rmtree(EXPORTED_SSH_KEYS_DIR)
+        shutil.rmtree(_exported_ssh_keys_dir())
     logger.info('Export complete!')
     logger.info(
         'You can import the profiles by running '
@@ -648,7 +652,7 @@ def import_profiles(archive_path, include_keys, logger):
                         "for one or more profiles. To restore those keys to "
                         "their original locations, you can use the "
                         "`--include-keys flag or copy them manually from %s ",
-                        EXPORTED_SSH_KEYS_DIR)
+                        _exported_ssh_keys_dir())
     logger.info('Import complete!')
     logger.info('You can list profiles using `cfy profiles list`')
 
@@ -689,11 +693,11 @@ def _move_ssh_key(profile, logger, is_backup):
     key_filepath = context.ssh_key
     if key_filepath:
         backup_path = os.path.join(
-            EXPORTED_SSH_KEYS_DIR, os.path.basename(key_filepath)) + \
+            _exported_ssh_keys_dir(), os.path.basename(key_filepath)) + \
             '.{0}.profile'.format(profile)
         if is_backup:
-            if not os.path.isdir(EXPORTED_SSH_KEYS_DIR):
-                os.makedirs(EXPORTED_SSH_KEYS_DIR, mode=0o700)
+            if not os.path.isdir(_exported_ssh_keys_dir()):
+                os.makedirs(_exported_ssh_keys_dir(), mode=0o700)
             logger.info('Copying ssh key %s to %s...',
                         key_filepath, backup_path)
             shutil.copy2(key_filepath, backup_path)
